"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const thrift_parser_1 = require("@creditkarma/thrift-parser");
const fs = tslib_1.__importStar(require("fs-extra"));
async function readCode(filefullname, options, // TODO 添加option支持
includeMap) {
    const source = await fs.readFile(filefullname);
    return parser(filefullname, source.toString(), options, includeMap);
}
exports.readCode = readCode;
function parser(filefullname, source, options, // TODO 添加option支持
includeMap) {
    const ast = thrift_parser_1.parse(source);
    if (!isThritDocument(ast)) {
        throw new Error('thrift parser error:' + filefullname);
    }
    const rtn = {
        ns: '',
        fileName: filefullname,
        includes: [],
        interfaces: [],
        enums: [],
        typeDefs: [],
        services: []
    };
    ast.body.forEach((ts) => {
        // namespace
        if (ts.type === thrift_parser_1.SyntaxType.NamespaceDefinition) {
            // namespace 的处理逻辑，抓一个就来了
            // TODO 优先考虑js的namespace，之后是go，再之后随便抓一个
            rtn.ns = ts.name.value;
        }
        // includes
        if (ts.type === thrift_parser_1.SyntaxType.IncludeDefinition) {
            rtn.includes.push(ts.path.value);
        }
        // struct like: struct exception union --> interfaces
        if (ts.type === thrift_parser_1.SyntaxType.StructDefinition ||
            ts.type === thrift_parser_1.SyntaxType.UnionDefinition ||
            ts.type === thrift_parser_1.SyntaxType.ExceptionDefinition) {
            const name = ts.name.value;
            const aInterface = {
                name,
                properties: {},
                childrenEnums: [],
                childrenInterfaces: []
            };
            // 添加属性
            ts.fields.forEach((field) => {
                const { entity: temp, name } = handleField(field, options);
                aInterface.properties[name] = temp;
            });
            rtn.interfaces.push(aInterface);
        }
        // typedef
        if (ts.type === thrift_parser_1.SyntaxType.TypedefDefinition) {
            const aTypeDef = { type: '', alias: '' };
            aTypeDef.alias = ts.name.value;
            aTypeDef.type = getFieldTypeString(ts.definitionType);
            rtn.typeDefs.push(aTypeDef);
        }
        // service
        if (ts.type === thrift_parser_1.SyntaxType.ServiceDefinition) {
            const aService = {
                name: ts.name.value,
                interfaces: {}
            };
            ts.functions.forEach(func => {
                aService.interfaces[func.name.value] = handleFunction(func, options);
            });
            rtn.services.push(aService);
        }
        // enum
        if (ts.type === thrift_parser_1.SyntaxType.EnumDefinition) {
            rtn.enums.push(handleEnum(ts));
        }
        // const 考虑支持
        if (ts.type === thrift_parser_1.SyntaxType.ConstDefinition) {
            //
        }
    });
    /* istanbul ignore if */
    if (includeMap) {
        includeMap[rtn.fileName] = rtn;
    }
    return rtn;
}
exports.parser = parser;
function isThritDocument(ast) {
    return ast.type === thrift_parser_1.SyntaxType.ThriftDocument;
}
/**
 *
 * @param {ThriftDocument} ast thrift-parser输出的ast
 * @returns {RpcEntity} 返回rpc
 */
function transformAst(ast) {
    return {};
}
exports.transformAst = transformAst;
function getFieldTypeString(fieldType) {
    const ThriftType2JavascriptType = {
        [thrift_parser_1.SyntaxType.BoolKeyword]: 'boolean',
        [thrift_parser_1.SyntaxType.ByteKeyword]: 'number',
        [thrift_parser_1.SyntaxType.I16Keyword]: 'number',
        [thrift_parser_1.SyntaxType.I32Keyword]: 'number',
        [thrift_parser_1.SyntaxType.I64Keyword]: 'Int64',
        [thrift_parser_1.SyntaxType.DoubleKeyword]: 'number',
        [thrift_parser_1.SyntaxType.StringKeyword]: 'string',
        [thrift_parser_1.SyntaxType.BinaryKeyword]: 'any',
        [thrift_parser_1.SyntaxType.ListKeyword]: 'Array',
        [thrift_parser_1.SyntaxType.MapKeyword]: 'Map',
        [thrift_parser_1.SyntaxType.SetKeyword]: 'Set',
        // UPDATE 添加void
        [thrift_parser_1.SyntaxType.VoidKeyword]: 'void'
    };
    if (fieldType.type === thrift_parser_1.SyntaxType.Identifier) {
        return fieldType.value;
    }
    if (fieldType.type === thrift_parser_1.SyntaxType.SetType) {
        return `Set<${getFieldTypeString(fieldType.valueType)}>`;
    }
    if (fieldType.type === thrift_parser_1.SyntaxType.ListType) {
        return `${getFieldTypeString(fieldType.valueType)}[]`;
    }
    if (fieldType.type === thrift_parser_1.SyntaxType.MapType) {
        return `Map<${getFieldTypeString(fieldType.keyType)}, ${getFieldTypeString(fieldType.valueType)}>`;
    }
    return ThriftType2JavascriptType[fieldType.type];
}
function handleComments(comments) {
    // TODO: comment parser的逻辑和正则的逻辑不能对齐，需要重新设计comnent的生成逻辑
    return '';
    return comments
        .map(c => {
        if (c.type === thrift_parser_1.SyntaxType.CommentLine) {
            return c.value;
        }
        else {
            return c.value.join('\n');
        }
    })
        .join('\n');
}
function handleField(field, options = {}) {
    let name = field.name.value;
    const comment = handleComments(field.comments);
    // 需要处理typedef
    const type = getFieldTypeString(field.fieldType);
    const index = field.fieldID ? field.fieldID.value : 0;
    const optional = options.useStrictMode
        ? field.requiredness !== 'required'
        : !(field.requiredness !== 'optional');
    // TODO 考虑多种type数据的default value StringLiteral | IntConstant | DoubleConstant | BooleanLiteral | ConstMap | ConstList | Identifier
    if (options.useTag) {
        const annotations = field.annotations;
        const tagValueReg = /json:\"([\w_\d]+).*\"/;
        const tagNameReg = /(\w+).tag/;
        if (annotations) {
            const nameTag = annotations.annotations.find((annotation) => {
                let match;
                if ((match = tagNameReg.exec(annotation.name.value))) {
                    return match[1] === options.useTag;
                }
                return false;
            });
            if (nameTag) {
                const match = tagValueReg.exec(nameTag.value ? nameTag.value.value : '');
                if (match) {
                    name = match[1];
                }
            }
        }
    }
    const defaultValue = '';
    return {
        entity: {
            type,
            index,
            optional,
            comment,
            defaultValue
        },
        name
    };
}
/*
  returnType: string;
  inputParams: Array<{
    type: string;
    index: number;
    name: string;
  }>;
  comment: string;
 */
function handleFunction(func, options) {
    const returnType = getFieldTypeString(func.returnType);
    const inputParams = func.fields.map((field) => {
        const { entity: temp, name } = handleField(field, options);
        return {
            type: temp.type,
            index: temp.index,
            name
        };
    });
    const comment = handleComments(func.comments);
    return {
        returnType,
        inputParams,
        comment
    };
}
function handleEnum(e, options) {
    const name = e.name.value;
    const properties = {};
    let currentValue = 0;
    e.members.forEach(member => {
        /**
         * https://wiki.apache.org/thrift/Tutorial
         * You can define enums, which are just 32 bit integers. Values are optional
         * and start at 1 if not supplied, C style again.
         *              ^ ThriftIDL page says "If no constant value is supplied,
         *   the value is either 0 for the first element, or one greater than the
         *   preceding value for any subsequent element" so I'm guessing that's a bug.
         *   PS: http://enel.ucalgary.ca/People/Norman/enel315_winter1997/enum_types/ states that if values are not supplied, they start at 0 and not 1.
         */
        let value;
        if (member.initializer) {
            value = parseInt(member.initializer.value.value, 10);
        }
        else {
            value = currentValue++;
        }
        properties[member.name.value] = {
            value,
            comment: handleComments(member.comments)
        };
    });
    return {
        name,
        properties
    };
}
